module Hashery
  VERSION = "2.1.0"
end

require 'hashery/core_ext'

#require 'hashery/basic_struct'
require 'hashery/casting_hash'
require 'hashery/crud_hash'
require 'hashery/dictionary'
require 'hashery/fuzzy_hash'
require 'hashery/ini_hash'
require 'hashery/linked_list'
require 'hashery/lru_hash'
require 'hashery/key_hash'
require 'hashery/open_cascade'
require 'hashery/open_hash'
#require 'hashery/ordered_hash'
#require 'hashery/ostructable'
require 'hashery/property_hash'
require 'hashery/query_hash'
require 'hashery/static_hash'


